/*
 * Copyright (C) 2004-2012 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef __CHMREFERENCECOLLECTION_H__
#define __CHMREFERENCECOLLECTION_H__

#include <string>
#include <set>
#include <queue>

class CHMReferenceCollection {

public:
	static std::string fullReference(const std::string &prefix, std::string reference);

public:
	CHMReferenceCollection();
	const std::string &addReference(const std::string &reference, bool doConvert);
	bool containsNonProcessedReferences() const;
	const std::string nextReference();
	void setPrefix(const std::string &fileName);
	const std::string &prefix() const;

private:
	std::string myPrefix;
	std::set<std::string> myReferences;
	std::queue<std::string> myReferenceQueue;

private:
	CHMReferenceCollection(const CHMReferenceCollection&);
	const CHMReferenceCollection &operator = (const CHMReferenceCollection&);
};

#endif /* __CHMREFERENCECOLLECTION_H__ */
