/*
 * Copyright (C) 2009-2012 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "../DBRunnables.h"
#include "../../../library/Book.h"

SaveBookRunnable::SaveBookRunnable(SaveTableBookRunnable &saveTableBook, SaveAuthorsRunnable &saveAuthors, 
		SaveSeriesRunnable &saveSeries, SaveTagsRunnable &saveTags) : 
	mySaveTableBook(saveTableBook), 
	mySaveAuthors(saveAuthors), 
	mySaveSeries(saveSeries),
	mySaveTags(saveTags) {
}

bool SaveBookRunnable::run() {
	return mySaveTableBook.run() 
		&& mySaveAuthors.run()
		&& mySaveSeries.run()
		&& mySaveTags.run();
}

void SaveBookRunnable::setBook(shared_ptr<Book> book) {
	mySaveTableBook.setBook(book);
	mySaveAuthors.setBook(book);
	mySaveTags.setBook(book);
	mySaveSeries.setBook(book);
}
