/*
 * Copyright (C) 2008-2012 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "ZLXMLNamespace.h"

const std::string ZLXMLNamespace::DublinCore = "http://purl.org/dc/elements/1.1/";
const std::string ZLXMLNamespace::DublinCoreLegacy = "http://purl.org/metadata/dublin_core";
const std::string ZLXMLNamespace::DublinCoreTerms = "http://purl.org/dc/terms/";
const std::string ZLXMLNamespace::XLink = "http://www.w3.org/1999/xlink";
const std::string ZLXMLNamespace::OpenPackagingFormat = "http://www.idpf.org/2007/opf";
const std::string ZLXMLNamespace::Atom = "http://www.w3.org/2005/Atom";
const std::string ZLXMLNamespace::OpenSearch = "http://a9.com/-/spec/opensearch/1.1/";
const std::string ZLXMLNamespace::CalibreMetadata = "http://calibre.kovidgoyal.net/2009/metadata";
const std::string ZLXMLNamespace::Opds = "http://opds-spec.org/2010/catalog";
const std::string ZLXMLNamespace::DaisyNCX = "http://www.daisy.org/z3986/2005/ncx/";
const std::string ZLXMLNamespace::FBReaderCatalogMetadata = "http://data.fbreader.org/catalog/metadata/";
